@@ -2,7 +2,12 @@ source common.sh
 
 clearStore
 
-export REMOTE_STORE=$TEST_ROOT/remote_store
+rm -f $TEST_ROOT/result
+
+export REMOTE_STORE=file:$TEST_ROOT/remote_store
+echo 'require-sigs = false' >> $NIX_CONF_DIR/nix.conf
+
+restartDaemon
 
 # Build the dependencies and push them to the remote store
 nix-build -o $TEST_ROOT/result dependencies.nix --post-build-hook $PWD/push-to-store.sh
